# Copyright (C) 2019-2021 HERE Europe B.V.
# SPDX-License-Identifier: Apache-2.0

"""This module contains configurations for various Location services end points."""

row_conf = dict(here_api="hereapi.com")

china_conf = dict(here_api="hereapi.cn")

conf = dict(row=row_conf, china=china_conf)
